% Visualizing the volumetric density-equalizing reference map (VDERM)
% results with the free boundary condition
% 
% If you use this code in your own work, please cite the following paper:
% [1] G. P. T. Choi and C. H. Rycroft,
%     "Volumetric density-equalizing reference map with applications."
%     Journal of Scientific Computing, 86(3), 41, 2021.
% 
% Copyright (c) 2021-2023, Gary P. T. Choi

%% Please first compile and run the cpp code
% g++ -I ./eigen-3.4.0/ -Ofast -fopenmp vderm_free.cpp -o vderm_free
% ./vderm_free

%% read the result obtained from the cpp program

mnl = textread('results/vderm_free_size.txt');
M = mnl(1);
N = mnl(2);
L = mnl(3);

initial_map = textread('results/vderm_free_initial_map.txt');
initial_map(:,1) = initial_map(:,1) - M/6;
initial_map(:,2) = initial_map(:,2) - N/6;
initial_map(:,3) = initial_map(:,3) - L/6;

initial_rho = textread('results/vderm_free_initial_rho.txt');
initial_rho = initial_rho(:,1);

reference_map = textread('results/vderm_free_final_map.txt');
reference_map(:,1) = reference_map(:,1) - M/6;
reference_map(:,2) = reference_map(:,2) - N/6;
reference_map(:,3) = reference_map(:,3) - L/6;

v = zeros(N,M,L,3);
v(:,:,:,1) = reshape(initial_map(:,1),N,M,L);
v(:,:,:,2) = reshape(initial_map(:,2),N,M,L);
v(:,:,:,3) = reshape(initial_map(:,3),N,M,L);
rho = reshape(initial_rho,N,M,L);

v_interior = v(N/6+1:5*N/6,M/6+1:5*M/6,L/6+1:5*L/6,:);
rho_interior = rho(N/6+1:5*N/6,M/6+1:5*M/6,L/6+1:5*L/6);

ref_map = zeros(N,M,L,3);
ref_map(:,:,:,1) = reshape(reference_map(:,1),N,M,L);
ref_map(:,:,:,2) = reshape(reference_map(:,2),N,M,L);
ref_map(:,:,:,3) = reshape(reference_map(:,3),N,M,L);

ref_map_interior = ref_map(N/6+1:5*N/6,M/6+1:5*M/6,L/6+1:5*L/6,:);

%% plot the initial state
plot_solid(v_interior, 3, 3, 3, rho_interior);
view([20 15])
title('Initial state')

%% plot the final state of the reference map (excluding the sea)
plot_solid(ref_map_interior, 3, 3, 3, rho_interior);
view([20 15])
title('Final state of the reference map')

%% plot the final state of the forward map (excluding the sea)
F1 = scatteredInterpolant(reference_map(:,1),reference_map(:,2),reference_map(:,3),initial_map(:,1),'linear');
F2 = scatteredInterpolant(reference_map(:,1),reference_map(:,2),reference_map(:,3),initial_map(:,2),'linear');
F3 = scatteredInterpolant(reference_map(:,1),reference_map(:,2),reference_map(:,3),initial_map(:,3),'linear');
X = F1(initial_map(:,1),initial_map(:,2),initial_map(:,3));
Y = F2(initial_map(:,1),initial_map(:,2),initial_map(:,3));
Z = F3(initial_map(:,1),initial_map(:,2),initial_map(:,3));

map = zeros(N,M,L,3);
map(:,:,:,1) = reshape(X,N,M,L);
map(:,:,:,2) = reshape(Y,N,M,L);
map(:,:,:,3) = reshape(Z,N,M,L);
map_interior = map(N/6+1:5*N/6,M/6+1:5*M/6,L/6+1:5*L/6,:);

plot_solid(map_interior,3,3,3,rho_interior);
view([20 15])
title('Final state of the forward map')
